//	RWARRAYS.C
//	Functions for reading and writing VB arrays to disk files.
//	Works with any array less than 64K in size and not containing
//	variable length data like dynamic strings or variants.

//	(c) Marcus Schommler, 1994
//		CServe: 100335,3620

#include <windows.h>
#include <vbapi.h>
#include <math.h>		// for abs()
#include <io.h> 		// for _filelength()

#define K64 65534
// Maximum number of bytes for a _lwrite buffer

#define SBUF 250
// Buffer size for file name buffer


LONG _far pascal _export GetElemCount(HAD hAD)
// Calculating the total number of elements in an array
// Is used internally but also exported for direct calls from VB
// hAD: 	Handle to array descriptor 
{
	LONG 	numElem;	// Number of array elements
	LONG 	lBounds; 	// For getting upper and lower bound of a dimension
	SHORT	dimens; 	// Number of dimensions for array
	SHORT 	i;
	
	dimens = VBArrayIndexCount(hAD);
	if (!dimens) return 0L; // quit if array is not valid
	
	numElem = 1L;
	for (i = 1; i <= dimens; i++) {
		lBounds = VBArrayBounds(hAD, i); // Get bound for this dimension
		numElem *= (abs(HIBOUND(lBounds) - LOBOUND(lBounds)) + 1); 
		// Calculate no. of elements for this dimension and multiply with
		// no. of elements in already processed dimensions
	}
	
	return numElem;
}


int _far pascal _export arr_write(HAD hAD, HLSTR hlsFile)
// Write an VB array to a disk file
// hAD: 	Handle to array descriptor 
// hlsFile: Handle for a VB string containing the file name
{
	HFILE 	outfile; 	// Handle for output file
	SHORT 	elemSize;   // Size of an array element
	LONG	numElem,	// Total number of elements in this array
	 	 	totalSize;	// Total array size in bytes
	char 	s[SBUF];	// Character buffer for file name
	SHORT 	i;			// Counter
	
	// Get size of an element for the array :
	if(!(elemSize = VBArrayElemSize(hAD))) return 0; 
	// quit if array is not valid
	
	// Get number of elements that can be stored in the array :
	if (!(numElem = GetElemCount(hAD))) return 0;
	totalSize = numElem * elemSize; // Multiply no. of elements with size
	
	if (totalSize > K64) return 0; // Array is too large 
	
	if (!VBGetHlstr(hlsFile, s, SBUF - 1)) return 0; // String is empty
	
	// Create the file ...
	outfile = _lcreat(s, 0);
	if (outfile == HFILE_ERROR) return 0;
	
	// ... and put out the data :
	i = _lwrite(outfile, VBArrayFirstElem(hAD), (UINT) totalSize);
	_lclose(outfile);
	
	// return either 0 or the number of elements written
	if (i == HFILE_ERROR) return 0; // error happened during write
	else return (int) numElem;
}


int _far pascal _export arr_read(HAD hAD, HLSTR hlsFile)
// Read an VB array from a disk file
// hAD: 	Handle to array descriptor 
// hlsFile: Handle for a VB string containing the file name

{
	HFILE 	infile;		// Handle for file to read from
	SHORT 	elemSize;   // Size of an array element
	LPVOID 	lpv;		// Pointer to VB array memory
	LONG	numElem,	// Total number of elements in this array
	 	 	fillen;     // Length of file
	char 	s[SBUF];	// Character buffer for file name
	UINT 	bytesRead;  // Number of bytes read from file
	
	// Get size of an element for the array :
	if(!(elemSize = VBArrayElemSize(hAD))) return 0; 

	// Get number of elements that can be stored in the array :
	if (!(numElem = GetElemCount(hAD))) return 0;
	
	
	if (!VBGetHlstr(hlsFile, s, SBUF - 1)) return 0; // String is empty
	
	infile = _lopen(s, READ); // Open the file
	if (infile == HFILE_ERROR) return 0; // File open error
	
	fillen = _filelength(infile);	// Get length of opened file
	
	// Checks for valid file length :
	if (!fillen || // file size is zero
		(fillen > K64) || // file is longer than 64K
		(numElem < fillen / elemSize)) // data from file won't fit
		return 0;

	// Get pointer to array location :	
	if (!(lpv = VBArrayFirstElem(hAD))) return 0; // Array is not valid

	// Read in data from file :
	bytesRead = _lread(infile, lpv, (UINT) fillen);
	_lclose(infile);
	
	// return either 0 or the number of elements read :
	if (bytesRead == HFILE_ERROR) return 0; // read error
	else return bytesRead / elemSize; 
	// return number of array elements read
}


int CALLBACK LibMain(hinst, wDataSeg, cbHeapSize, lpszCmdLine)
{
	return 1;
}
